/*_______________________________________________________________________________________________________________________________________________________________________
SNS_ObjectLibrary.mq4 

Version 2.0 11/29/09 
Copyright (C) 2009, Steven N. Scott  aka "SNSCOTT"
mailto:steven.n.scott@gmail.com
________________________________________________________________________________________________________________________________________________________________________
VERSION HISTORY

v1.0			"Initial" Release [ when I started to keep better track of versions :-)	]					11/07/2009
v1.1			Some minor bug fixes and added Draw_FiboChannel														11/09/2009
v1.4			Added options to make FIBO's draw as RAYS																11/12/2009
v2.0			Several Changes/additions as well as renaming of functions to be more consistent			11/29/2009


                                                                 
_______________________________________________________________________________________________________________________________________________________________________*/


#define version "2.0 : 11/29/2009"


#property copyright "Steven N. Scott"
#property link      "steven.n.scott@gmail.com"
#property library


//=====================================================================================================================================================================
//=====================================================================================================================================================================
//=====================================================================================================================================================================
// GENERAL UTILITIES


	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// SNSLIBRARY.VERSION
	// Returns version number and date for this instance of the library
	
	string	SNSLibrary.Version(){ 
	
		return(version); 
		
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// SNSTAG
	// Draws the SNS INDICATORS "signature" tag unobtrusively in the lower-left corner of the chart

	void SNSTAG(){

		Draw_Label("SNS", "TAG", "sns", 2, 1, 1, "Tahoma", 12, MediumOrchid, false, 0);
	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DELETEOBJECTSBYPREFIX
	// Deletes all objects with a given prefix to their name

	void DeleteObjectsByPrefix(string Prefix) {
	
   	int L = StringLen(Prefix);
   	int i = 0; 
   	while(i < ObjectsTotal()) {
   
       	string ObjName = ObjectName(i);
       	if(StringSubstr(ObjName, 0, L) != Prefix) { 	i++; 	continue;  }
         
       	ObjectDelete(ObjName);
       
     	}
     
	}

	bool   IsMini()         { return(StringFind(Symbol(),"m")  > -1); }

	bool   IsDoubleDotted() { return(StringFind(Symbol(),"..") > -1); }

	bool   IsDotted()       { return(StringFind(Symbol(),".")  > -1); }
	

//=====================================================================================================================================================================
//=====================================================================================================================================================================
//=====================================================================================================================================================================

// GRAPHIC OBJECT UTILITIES


	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_HLINE
	// Draw a HORIZONTAL LINE OBJECT at the provided PRICE

	void Draw_HLine(string prefix, string name, int style, int width, color clr, datetime x, double y, bool back, int window=0){
	
  		name = prefix+name;	
  		if (ObjectFind(name)<0) ObjectCreate (name,OBJ_HLINE, window, x, y); else if (x!=0) ObjectMove(name,window, x,y );
  		ObjectSet(name, OBJPROP_STYLE, style);
  		ObjectSet(name, OBJPROP_BACK, back);
  		ObjectSet(name, OBJPROP_WIDTH, width);
  		ObjectSet(name, OBJPROP_COLOR,clr );

  
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_VLINE
	// Draw a HORIZONTAL LINE OBJECT at the provided DATETIME

	void Draw_VLine(string prefix, string name, int style, int width, color clr, datetime x, bool back, int window=0){
	
  		name = prefix+name;	
  		if (ObjectFind(name)<0) ObjectCreate (name,OBJ_VLINE, window, x, 0); else if (x!=0) ObjectMove(name,window, x,0 );
  		ObjectSet(name, OBJPROP_STYLE, style);
  		ObjectSet(name, OBJPROP_BACK, back);
  		ObjectSet(name, OBJPROP_WIDTH, width);
  		ObjectSet(name, OBJPROP_COLOR, clr );

	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_TLINE
	// Draws a TRENDLINE OBJECT from a given DATETIME/PRICE to another DATETIME/PRICE

	void Draw_TLine(string prefix, string name, int style, int width, color clr, datetime x1, double y1, datetime x2, double y2, bool ray, bool back, int window=0){
	
  		name = prefix+name;	
  		if (ObjectFind(name)<0) ObjectCreate (name,OBJ_TREND, window, x1, y1, x2, y2); else {
   
   			if (x1!=0 && x2!=0) {
   			
      			ObjectMove(name,0,x1,y1);
      			ObjectMove(name,1,x2,y2);
      			
      		}
      
  		}
  		
  		ObjectSet(name, OBJPROP_STYLE, style);
  		ObjectSet(name, OBJPROP_BACK, back);
  		ObjectSet(name, OBJPROP_WIDTH, width);
  		ObjectSet(name, OBJPROP_COLOR, clr );
  		ObjectSet(name, OBJPROP_RAY, ray);
  
	}
	
	
	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_LABELEDLINE
	// Draws a TRENDLINE OBJECT with a LABEL nearby
	
	void Draw_LabeledLine(string prefix, string name, int style, int width, color clr, datetime x1, double y1, datetime x2, double y2, bool ray, 
								 string label, string font, int fontsize, color fontcolor, datetime labelx, double labely, int window=0){

		Draw_TLine(prefix, name, style, width, clr, x1, y1, x2, y2, ray, true, window);
	   Draw_Text(prefix, name+".label", label, labelx, y1+labely, font, fontsize, fontcolor, false, window);
  
	}
	

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_BOX
	// Draws a RECTANGLE OBJECT at the given datetime/price coordinates
	
	void Draw_Box(string prefix, string name, color boxcolor, datetime x1, double y1, datetime x2, double y2, bool back,int linewidth,int linestyle, int window=0){

   	name = prefix+name;	
   	if (ObjectFind(name)<0) ObjectCreate(name, OBJ_RECTANGLE,window, x1,y1, x2,y2); else {
   
   			if (x1!=0 && x2!=0) {
   			
      			ObjectMove(name,0,x1,y1);
      			ObjectMove(name,1,x2,y2);
      			
      		}
            
   	}
   
   	ObjectSet(name, OBJPROP_COLOR, boxcolor);
   	ObjectSet(name,OBJPROP_BACK,back);
   	ObjectSet(name,OBJPROP_WIDTH,linewidth);
   	ObjectSet(name,OBJPROP_STYLE,linestyle);

	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_FIBO
	// Draws a FIBONACCI OBJECT
	
	void Draw_Fibo(string prefix, string name, int style, int width, color fibcolor, int levelstyle, int levelwidth, color levelcolor,  datetime x1, double y1, datetime x2, double y2, bool ray, bool back, int window=0){

		int x,i;
		name = prefix+name;	
   	if (ObjectFind(name)<0) ObjectCreate(name, OBJ_FIBO,window, x1,y1, x2,y2); else {
   
   			if (x1!=0 && x2!=0) {
   			
      			ObjectMove(name,0,x1,y1);
      			ObjectMove(name,1,x2,y2);
      			
      		}
            
   	}
   
   	ObjectSet(name, OBJPROP_STYLE, style);
   	ObjectSet(name, OBJPROP_WIDTH, width);
   	ObjectSet(name, OBJPROP_COLOR, fibcolor);
   	
   	ObjectSet(name, OBJPROP_LEVELSTYLE, levelstyle);
   	ObjectSet(name, OBJPROP_LEVELWIDTH, levelwidth);
   	ObjectSet(name, OBJPROP_LEVELCOLOR, levelcolor);
   	
   	ObjectSet(name, OBJPROP_RAY, ray);
	  	ObjectSet(name, OBJPROP_BACK,back);
   	
   	ObjectSet(name, OBJPROP_FIBOLEVELS, 6);
   	
      x=OBJPROP_FIRSTLEVEL; i=0;
      ObjectSet(name,x,0.236);	ObjectSetFiboDescription(name,i,"");		i++; x=OBJPROP_FIRSTLEVEL+i; 
      ObjectSet(name,x,0.382);	ObjectSetFiboDescription(name,i,"");		i++; x=OBJPROP_FIRSTLEVEL+i; 
      ObjectSet(name,x,0.500);	ObjectSetFiboDescription(name,i,"50%");	i++; x=OBJPROP_FIRSTLEVEL+i; 
      ObjectSet(name,x,0.618);	ObjectSetFiboDescription(name,i,"");		i++; x=OBJPROP_FIRSTLEVEL+i; 
      ObjectSet(name,x,0.764);	ObjectSetFiboDescription(name,i,"");		i++; x=OBJPROP_FIRSTLEVEL+i; 
      ObjectSet(name,x,1.0);		ObjectSetFiboDescription(name,i,"100%");	i++; x=OBJPROP_FIRSTLEVEL+i; 

	}


	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_FIBOCHANNEL
	// Draws a FIBONACCI CHANNEL OBJECT
	
	void Draw_FiboChannel(string prefix, string name, int style, int width, color fibcolor, int levelstyle, int levelwidth, color levelcolor,  datetime x1, double y1, datetime x2, double y2, datetime x3, double y3, bool ray, bool back, int window=0){

		int x,i;
		name = prefix+name;	
   	if (ObjectFind(name)<0) ObjectCreate(name, OBJ_FIBOCHANNEL, window, x1,y1, x2,y2, x3,y3); else {
   
   			if (x1!=0 && x2!=0 && x3!=0) {
   			
      			ObjectMove(name,0,x1,y1);
      			ObjectMove(name,1,x2,y2);
      			ObjectMove(name,2,x3,y3);
      			
      		}
            
   	}
   
   	ObjectSet(name, OBJPROP_STYLE, style);
   	ObjectSet(name, OBJPROP_WIDTH, width);
   	ObjectSet(name, OBJPROP_COLOR, fibcolor);
   	
   	ObjectSet(name, OBJPROP_LEVELSTYLE, levelstyle);
   	ObjectSet(name, OBJPROP_LEVELWIDTH, levelwidth);
   	ObjectSet(name, OBJPROP_LEVELCOLOR, levelcolor);
   	
   	ObjectSet(name, OBJPROP_RAY, ray);
	  	ObjectSet(name, OBJPROP_BACK,back);
   	
   	ObjectSet(name, OBJPROP_FIBOLEVELS, 23);
      
      x=OBJPROP_FIRSTLEVEL; i=0;
      ObjectSet(name,x,0.236);	ObjectSetFiboDescription(name,i,"0.236");  i++; x=OBJPROP_FIRSTLEVEL+i;   
      ObjectSet(name,x,0.382);	ObjectSetFiboDescription(name,i,"0.382");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,0.500);	ObjectSetFiboDescription(name,i,"0.500");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,0.618);	ObjectSetFiboDescription(name,i,"0.618");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,0.764);	ObjectSetFiboDescription(name,i,"0.764");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,1.0);		ObjectSetFiboDescription(name,i,"1.000");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,1.236);	ObjectSetFiboDescription(name,i,"1.236");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,1.382);	ObjectSetFiboDescription(name,i,"1.382");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,1.618);	ObjectSetFiboDescription(name,i,"1.500");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,1.618);	ObjectSetFiboDescription(name,i,"1.618");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,1.764);	ObjectSetFiboDescription(name,i,"1.764");  i++; x=OBJPROP_FIRSTLEVEL+i; 	

      ObjectSet(name,x,0);			ObjectSetFiboDescription(name,i,"0"); 		 i++; x=OBJPROP_FIRSTLEVEL+i; 	
      
      ObjectSet(name,x,-0.236); 	ObjectSetFiboDescription(name,i,"-0.236"); i++; x=OBJPROP_FIRSTLEVEL+i;  
      ObjectSet(name,x,-0.382);	ObjectSetFiboDescription(name,i,"-0.382"); i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,-0.500);	ObjectSetFiboDescription(name,i,"-0.500"); i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,-0.618);	ObjectSetFiboDescription(name,i,"-0.618"); i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,-0.764);	ObjectSetFiboDescription(name,i,"-0.764"); i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,-1.0);		ObjectSetFiboDescription(name,i,"-1.000"); i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,-1.236);	ObjectSetFiboDescription(name,i,"-1.236");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,-1.382);	ObjectSetFiboDescription(name,i,"-1.382"); i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,-1.500);	ObjectSetFiboDescription(name,i,"-1.500");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,-1.618);	ObjectSetFiboDescription(name,i,"-1.618");  i++; x=OBJPROP_FIRSTLEVEL+i; 	
      ObjectSet(name,x,-1.764);	ObjectSetFiboDescription(name,i,"-1.764");  i++; x=OBJPROP_FIRSTLEVEL+i; 	

	}


	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_TEXT
	// Draws a TEXT OBJECT at given datetime/price coordinates
	
	void Draw_Text(string prefix, string name, string text, datetime x, double y, string font, int fontsize, color fontcolor, bool back, int window=0){

  		name = prefix+name;	
  		if (ObjectFind(name)<0) ObjectCreate(name, OBJ_TEXT, window, x,y); else if (x!=0) ObjectMove(name,0, x,y );
  		ObjectSet(name,OBJPROP_BACK, back); 
  		ObjectSetText(name, text , fontsize, font, fontcolor);

	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_VTEXT
	// Draws VERTICAL TEXT at the given datetime/price coordinates
	
	void Draw_VText(string prefix, string name, string text, datetime x, double y, string font, int fontsize, color fontcolor, bool back, int window=0){

  		name = prefix+name;	
  		if (ObjectFind(name)<0) ObjectCreate(name, OBJ_TEXT, window, x,y); else if (x!=0) ObjectMove(name,0, x,y );
  		ObjectSet(name,OBJPROP_BACK, back); 
  		ObjectSet(name,OBJPROP_ANGLE,90);
  		ObjectSetText(name, text , fontsize, font, fontcolor);

	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_HIGHLIGHTEDTEXT
	// Draws a TEXT OBJECT at given datetime,price coordinates with a RECTANGLE OBJECT behind it to provide a different colored backgroun for TEXT

	void Draw_HighLightedText(string prefix, string name, string text, datetime x, double y, string font, int fontsize, color fontcolor, color highcolor, int w, double h, int window=0){

		Draw_Box(prefix, name+"hl", highcolor, x, y, Time.Add(y,w), y+h, true,1, 0);
		Draw_Text(prefix, name, text, x, y, font, fontsize, fontcolor, false, window);

	}



	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_LABEL
	// Draws a LABEL OBJECT at given x.y cooridnates based on given CORNER and WINDOW
	
	void Draw_Label(string prefix, string name, string text, int corner, int x, int y, string font, int fontsize, color fontcolor, bool back, int window=0){

  		name = prefix+name;	
  		if (ObjectFind(name)<0) ObjectCreate(name, OBJ_LABEL, window, 0, 0); 
  		ObjectSetText(name, text, fontsize, font, fontcolor );
  		ObjectSet(name, OBJPROP_CORNER, corner);
  		ObjectSet(name, OBJPROP_BACK, back);
  		ObjectSet(name, OBJPROP_XDISTANCE, x);
  		ObjectSet(name, OBJPROP_YDISTANCE, y);
  
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_SYMBOL
	// Draws an ARROWOBJECT using the specified ARROWCODE
	
	void Draw_Symbol(string prefix, string name, int code, color colour, datetime x, double y, int size, bool back, int window=0){

   	name = prefix+name;	
   	if (ObjectFind(name)<0) ObjectCreate(name,OBJ_ARROW,window,x,y); else ObjectMove(name,0, x,y );

   	ObjectSet(name,OBJPROP_ARROWCODE,code);
   	ObjectSet(name,OBJPROP_COLOR,colour);
   	ObjectSet(name,OBJPROP_WIDTH,size);
   
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAWPRICEBOX
	// Draws a standard PRICEARROW OBJECT at the given datetime / price coordinates
	
	void Draw_PriceBox(string prefix, string name, bool left, int width, color colour, datetime x, double y, bool back, int window=0){

	 	int side;
	 	if (left) side=SYMBOL_LEFTPRICE; else side=SYMBOL_RIGHTPRICE;
	  
    	name = prefix+name;	
    	if (ObjectFind(name)<0) ObjectCreate(name,OBJ_ARROW,window,x,y); else {
   
      	ObjectMove(name,window,x,y);
      
    	}
    	ObjectSet(name,OBJPROP_ARROWCODE,side);
    	ObjectSet(name,OBJPROP_COLOR,colour);  
    	ObjectSet(name,OBJPROP_WIDTH,width);
   
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// GETARROW
	// Returns the WingDings arrow character for the given direction

	int GetArrow(string dir){

		dir=String.UpperCase(dir);
		
   	if(dir== "NEUTRAL" 	|| dir=="EQ")		return(159);
   	if(dir=="UP" 			|| dir=="N")      return(233);
   	if(dir=="UPLEFT" 		|| dir=="NW")     return(235);
   	if(dir=="UPRIGHT"		|| dir=="NE")     return(236);

   	if(dir=="DOWN"			|| dir=="S")      return(234);
   	if(dir=="DOWNLEFT"	|| dir=="SW")     return(237);
   	if(dir=="DOWNRIGHT"	|| dir=="SE")    	return(238);

   	if(dir=="RIGHT" 		|| dir=="E")      return(232);
   	if(dir=="LEFT"			|| dir=="W")      return(231);

   	return(234);
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// DRAW_ARROW
	// Draws an arrow based on the passed in DIRection string [N,S,E,W,NW,NE,SW,SE] or [UP,DOWN,LEFT,RIGHT,UPLEFT,UPRIGHT,DOWNLEFT,DOWNRIGHT]
	
	void Draw_Arrow(string prefix, string name, string dir, color colour, datetime x, double y, int size, bool back, int window=0){ 

    	Draw_Symbol(prefix, name, GetArrow(dir), colour, window, x, y, size, back );

	}


//=====================================================================================================================================================================
//=====================================================================================================================================================================
//=====================================================================================================================================================================

// DATE AND TIME UTILITIES

	string months[13]={"","JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"};



	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// TIME.ADDPERIOD
	// Add a given number of SPECIFIED periods to a date/time

	datetime Time.AddPeriod(datetime dt, int period, int increment) {

		return (dt+(D'1970.01.01 00:01:00'*(increment*period)));
	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// TIME.ADDMINUTES
	// Add a given number of MINUTES to a date/time

	datetime Time.AddMinutes(datetime dt, int minutes) {

		return (Time.AddPeriod(dt, PERIOD_M1, minutes));
	
	}

	//---------------------------------------------------------------------------------------------------------------
	// TIME.ADDHOURS
	// Add a given number of HOURS to a date/time

	datetime Time.AddHours(datetime dt, int hours) {

		return (Time.AddPeriod(dt, PERIOD_H1, hours));
	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// TIME.ADDDAYS
	// Add a given number of DAYS to a date/time

	datetime Time.AddDays(datetime dt, int days) {

		return (Time.AddPeriod(dt, PERIOD_H1, days));
	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// TIME.ADD
	// Add a given number of [current chart time period]'s to a date/time

	datetime Time.Add(datetime dt, int increment) {

		return (Time.AddPeriod(dt, Period(), increment));
	
	}


	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// TIME.MDYHMSTOSTR
	// Converts the individually passed in values for the date and time into the text representation of that date and time
	string Time.MDYHMSToStr(int nYear,int nMonth,int nDay,int nHour,int nMin,int nSec) {

   	string sMonth,sDay,sHour,sMin,sSec;

   	sMonth=100+nMonth;
   	sMonth=StringSubstr(sMonth,1);
   	sDay=100+nDay;
   	sDay=StringSubstr(sDay,1);
   	sHour=100+nHour;
   	sHour=StringSubstr(sHour,1);
   	sMin=100+nMin;
   	sMin=StringSubstr(sMin,1);
   	sSec=100+nSec;
   	sSec=StringSubstr(sSec,1);

   	return(StringConcatenate(nYear,".",sMonth,".",sDay," ",sHour,":",sMin,":",sSec));
   
	}


	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// TIME.DATETIMETOSTR
	// Converts datetime variable into text representation of the date and time
	
	string Time.DateTimeToStr(datetime dt) {

		return( Time.MDYHMSToStr(TimeYear(dt),TimeMonth(dt),TimeDay(dt),TimeHour(dt),TimeMinute(dt),TimeSeconds(dt) ) );

	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// NOTE [Time.ToFormatStr] IS NOT WORKING COMPLETELY CORRECTLY YET (AT LEAST THE WAY I WANT IT TO)
	// example "DOM ;MON ;HHH:;MMM "
	// DOM - DAY OF MONTH (1-31)
	// DOW - DAY OF WEEK (MON-SUN)
	// MON - MONTH (JAN-DEC)
	// HHH  - HOUR (00-23)
	// MMM  - MINUTE (00-59)
	// YYY  - YEAR (last 2 digits)

	string Time.ToFormatStr(datetime dt, string format) {
	string element[],dstr="unk",e,sep,hr,m;

		int elements = String.ToUCArray(element,format);
		
		for(int i=0; i<elements; i++) {
		
			e=element[i];
			Comment("element[0]='" + element[0] + "'  element[1]='" + element[1]+"'");
			if(StringLen(e)>3) sep=StringSubstr(e,3); else sep="";
			e=StringSubstr(e,0,3);
			hr=""+TimeHour(dt); if(StringLen(hr)<2) hr="0"+hr;
			m=""+TimeMinute(dt); if(StringLen(m)<2) m="0"+m;
			if (e=="DOM") { dstr=dstr+TimeDay(dt)+sep; continue;				}
			if (e=="MON") { dstr=dstr+months[TimeMonth(dt)]+sep; continue;	}
			if (e=="HHH") { dstr=dstr+hr+sep; continue;							}
			if (e=="MMM") { dstr=dstr+m+sep; continue;							}
		
		}
		
		return( dstr );

	}	
	


	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// TIME.TFTOSTRING
	// Converts a given MT4 Time period into the shortened textual name of that period
	
	string Time.TFToString(int tf){

   	string tfs;
   	switch(tf) {
   
      	case PERIOD_M1:  tfs="M1"  ; break;
      	case PERIOD_M5:  tfs="M5"  ; break;
      	case PERIOD_M15: tfs="M15" ; break;
      	case PERIOD_M30: tfs="M30" ; break;
      	case PERIOD_H1:  tfs="H1"  ; break;
      	case PERIOD_H4:  tfs="H4"  ; break;
      	case PERIOD_D1:  tfs="D1"  ; break;
      	case PERIOD_W1:  tfs="W1"  ; break;
      	case PERIOD_MN1: tfs="MN"	; break;
      	default: tfs=""+tf;
      
   	}
   
   	return(tfs);
   
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// TIME.FRAMEUNIT
	// Converts a given time frame into the textual name for that unit of Time
	
	string Time.FrameUnit(int tf){

   	string tfs;
   	if(tf==0) tf=Period();
   	switch(tf) {
   
      	case PERIOD_M1:  tfs="Minute"; 		break;
      	case PERIOD_M5:  tfs="5-Minute"; 	break;
      	case PERIOD_M15: tfs="15-Minute"; 	break;
      	case PERIOD_M30: tfs="30-Minute"; 	break;
      	case PERIOD_H1:  tfs="Hour"; 			break;
      	case PERIOD_H4:  tfs="4-Hour"; 		break;
      	case PERIOD_D1:  tfs="Day"; 			break;
      	case PERIOD_W1:  tfs="Week"; 			break;
      	case PERIOD_MN1: tfs="Month";
      
   	}
   
   	return(tfs);
   
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// TIME.ENCOMPASSING
	// Returns a "reasonable" time period that encompasses the given time period

	int Time.Encompassing(string tf){
	
		tf= StringTrimLeft(StringTrimRight(String.UpperCase(tf)));
		
   	if (tf=="M1"  || tf=="1") 						return(PERIOD_M15);	else
   	if (tf=="M5"  || tf=="5")						return(PERIOD_H1);  	else
   	if (tf=="M15" || tf=="15")						return(PERIOD_H1);	else
   	if (tf=="M30" || tf=="30")						return(PERIOD_D1);  	else
   	if (tf=="H1"  || tf=="H" || tf=="60") 		return(PERIOD_D1);   else
   	if (tf=="H4"  || tf=="4" || tf=="240") 	return(PERIOD_D1);   else
   	if (tf=="D1"  || tf=="D" || tf=="1440") 	return(PERIOD_W1);   else
   	if (tf=="W1"  || tf=="W" || tf=="10080") 	return(PERIOD_MN1);  else
	
		return(0);
	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// TIME.DOW
	// Returns the full name of the day of the week

	string	Time.DOW(int d) {
	
		switch(d) {
		
			case 0 : return("Sunday");
			case 1 : return("Monday");
			case 2 : return("Tuesday");
			case 3 : return("Wednesday");
			case 4 : return("Thursday");
			case 5 : return("Friday");
			case 6 : return("Saturday");
			
	   }
	
		return("Caturday :-)");  // for LOLCat fans
	
	}


//=====================================================================================================================================================================
//=====================================================================================================================================================================
//=====================================================================================================================================================================

// STRING UTILITIES

  	string lower  = "abcdefghijklmnopqrstuvwxyz";
  	string upper  = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";


	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.TOTIMEFRAME
	// Converts the typical user-entered values for a time period into the numeric equivalent MT4 uses internally

	int String.ToTimeFrame(string tf){
	
		tf= StringTrimLeft(StringTrimRight(String.UpperCase(tf)));
		
   	if (tf==""    || tf=="0") 													return(Period()); 	else
   	if (tf=="PERIOD_M1"  || tf=="M1"  || tf=="1") 						return(PERIOD_M1); 	else
   	if (tf=="PERIOD_M5"  || tf=="M5"  || tf=="5")						return(PERIOD_M5); 	else
   	if (tf=="PERIOD_M15" || tf=="M15" || tf=="15")						return(PERIOD_M15);	else
   	if (tf=="PERIOD_M30" || tf=="M30" || tf=="30")						return(PERIOD_M30);  else
   	if (tf=="PERIOD_H1"  || tf=="H1"  || tf=="H" || tf=="60") 		return(PERIOD_H1);   else
   	if (tf=="PERIOD_H4"  || tf=="H4"  || tf=="4" || tf=="240") 		return(PERIOD_H4);   else
   	if (tf=="PERIOD_D1"  || tf=="D1"  || tf=="D" || tf=="1440") 	return(PERIOD_D1);   else
   	if (tf=="PERIOD_W1"  || tf=="W1"  || tf=="W" || tf=="10080") 	return(PERIOD_W1);   else
   	if (tf=="PERIOD_MN1" || tf=="MN1" || tf=="M" || tf=="43200") 	return(PERIOD_MN1); 
	
		return(0);
	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.REPEAT
	// Repeats the string STR N times.

	string String.Repeat(string str, int n){
		
  		string outstr = "";
  		for(int i=0; i<n; i++) outstr = outstr + str;
  		return(outstr);
  	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.UPPERCASE
	// Returns string converted to UPPER CASE

	string String.UpperCase(string str){

  		string outstr = "";
  		for(int i=0; i<StringLen(str); i++)  {
    		int t1 = StringFind(lower,StringSubstr(str,i,1),0);
    		if (t1 >=0)  
      		outstr = outstr + StringSubstr(upper,t1,1);
    		else
      		outstr = outstr + StringSubstr(str,i,1);
  		}
  		
  		return(outstr);
  		
	}  

	
	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.LOWERCASE
	// Returns string converted to LOWECASE

	string String.LowerCase(string str){
	
  		string outstr = "";
  		for(int i=0; i<StringLen(str); i++)  {
    		int t1 = StringFind(upper,StringSubstr(str,i,1),0);
    		if (t1 >=0)  
      		outstr = outstr + StringSubstr(lower,t1,1);
    		else
      		outstr = outstr + StringSubstr(str,i,1);
  		}
  
  		return(outstr);
  
	}	

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.LEFTPAD
	// Prepends occurrences of the string STR2 to the string STR to make a string N characters long
	// Usage:    string x=StringLeftPad("ABCDEFG",9," ")  returns x = "  ABCDEFG"

	string String.LeftPad(string str, int n, string str2) {
		
  		return(String.Repeat(str2,n-StringLen(str)) + str);
  	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.RIGHTPAD
	// Appends occurrences of the string STR2 to the string STR to make a string N characters long
	// Usage:    string x=StringRightPad("ABCDEFG",9," ")  returns x = "ABCDEFG  "

	string String.RightPad(string str, int n, string str2)	{
		
  		return(str + String.Repeat(str2,n-StringLen(str)));
  	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.LEFT
	// if N is positive - returns the leftmost N characters of STR.          Usage: string x=StringRepeat("ABCDEFG",2)  returns x = "AB"
	// if N is negative - returns all but the rightmost N characters of STR. Usage:  string x=StringRepeat("ABCDEFG",-2)  returns x = "ABCDE"	

	string String.Left(string str, int n){
	
  		if (n > 0)  return(StringSubstr(str,0,n));
  		if (n < 0)  return(StringSubstr(str,0,StringLen(str)+n));
  		return("");
  		
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.RIGHT
	// if N is positive - returns the rightmost N characters of STR.        Usage: string x=StringRepeat("ABCDEFG",2)  returns x = "FG"
	// if N is negative - returns all but the leftmost N characters of STR. Usage: string x=StringRepeat("ABCDEFG",-2)  returns x = "CDEFG"

	string String.Right(string str, int n){
	
  		if (n > 0)  return(StringSubstr(str,StringLen(str)-n,n));
  		if (n < 0)  return(StringSubstr(str,-n,StringLen(str)-n));
  		return("");
  	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.TOARRAY
	// Converts a delimited string into individual array elements with no modification to string

	int String.ToArray(string& array[],string source){

   	source = StringTrimLeft(StringTrimRight(source));
   	if (StringSubstr(source,StringLen(source)-1,1) != ";")
                    	source = StringConcatenate(source,";");

   	int  s = 0;
   	int  i = StringFind(source,";",s);
   	string current;
      	while (i > 0)
      	{
         	current = StringSubstr(source,s,i-s);
         	ArrayResize(array,ArraySize(array)+1);
                     	array[ArraySize(array)-1] = current;
                     	s = i + 1;
                     	i = StringFind(source,";",s);
      	}
   	return(ArraySize(array));
	}

	
	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.TOUCARRAY
	// Converts a delimited string into individual array elements with string values converted to UPPER CASE

	int String.ToUCArray(string& array[],string source){

   	source = String.UpperCase(source);
   	String.ToArray(array,source);
   	return(ArraySize(array));

	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.TOLCARRAY
	// Converts a delimited string into individual array elements with string values converted to LOWER CASE

	int String.ToLCArray(string& array[],string source){

   	source = String.LowerCase(source);
   	String.ToArray(array,source);
   	return(ArraySize(array));

	}


	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.TOINTARRAY
	// Converts a delimited string into individual array elements with string values converted to INTEGERS
	
	int String.ToIntArray(int& array[],string source){

   	source = StringTrimLeft(StringTrimRight(source));
   	if (StringSubstr(source,StringLen(source)-1,1) != ";") source = StringConcatenate(source,";");

   	int  s = 0;
   	int  i = StringFind(source,";",s);
   	string current;
   	while (i > 0)	{
   	
      	current = StringSubstr(source,s,i-s);
      	ArrayResize(array,ArraySize(array)+1);
      	array[ArraySize(array)-1] = StrToInteger(current);
      	s = i + 1;
      	i = StringFind(source,";",s);
                  	
   	}
   	
   	return(ArraySize(array));
   	
	}  
	

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.TODOUBLEARRAY
	// Converts a delimited string into individual array elements with string values converted to INTEGERS
	
	int String.ToDoubleArray(int& array[],string source){

   	source = StringTrimLeft(StringTrimRight(source));
   	if (StringSubstr(source,StringLen(source)-1,1) != ";") source = StringConcatenate(source,";");

   	int  s = 0;
   	int  i = StringFind(source,";",s);
   	string current;
   	while (i > 0)	{
   	
      	current = StringSubstr(source,s,i-s);
      	ArrayResize(array,ArraySize(array)+1);
      	array[ArraySize(array)-1] = NormalizeDouble(StrToDouble(current),Digits);
      	s = i + 1;
      	i = StringFind(source,";",s);
                  	
   	}
   	
   	return(ArraySize(array));
   	
	}  
	

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.TRIM
	// Returns string with trailing spaces removed

	string String.Trim(string str)	{
	
  		string outstr = "";
  		for(int i=0; i<StringLen(str); i++)  if (StringSubstr(str,i,1) != " ")		outstr = outstr + StringSubstr(str,i,1);
  		return(outstr);
  		
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// STRING.RIGHTALIGN
	// Returns string with spaces padding to left to make it align to the right of a given field size

	string String.RightAlign( string rString, int ra ){
 
 		string FILLED = String.LeftPad(rString,ra," ");
 		int sl = StringLen(FILLED) ;
		return(StringSubstr(FILLED, sl-ra+1  , ra)) ;
		
	} 

	//-----------------------------------------------------------------------------------------------------------------------------------------------------


	int String.ToColor(string str) {

  	str = String.LowerCase(str);
  	if (str == "aliceblue")              return(0xFFF8F0);
  	if (str == "antiquewhite")           return(0xD7EBFA);
  	if (str == "aqua")                   return(0xFFFF00);
  	if (str == "aquamarine")             return(0xD4FF7F);
  	if (str == "beige")                  return(0xDCF5F5);
  	if (str == "bisque")                 return(0xC4E4FF);
  	if (str == "black")                  return(0x000000);
  	if (str == "blanchedalmond")         return(0xCDEBFF);
  	if (str == "blue")                   return(0xFF0000);
  	if (str == "blueviolet")             return(0xE22B8A);
  	if (str == "brown")                  return(0x2A2AA5);
  	if (str == "burlywood")              return(0x87B8DE);
  	if (str == "cadetblue")              return(0xA09E5F);
  	if (str == "chartreuse")             return(0x00FF7F);
  	if (str == "chocolate")              return(0x1E69D2);
  	if (str == "coral")                  return(0x507FFF);
  	if (str == "cornflowerblue")         return(0xED9564);
  	if (str == "cornsilk")               return(0xDCF8FF);
  	if (str == "crimson")                return(0x3C14DC);
  	if (str == "darkblue")               return(0x8B0000);
  	if (str == "darkgoldenrod")          return(0x0B86B8);
  	if (str == "darkgray")               return(0xA9A9A9);
  	if (str == "darkgreen")              return(0x006400);
  	if (str == "darkkhaki")              return(0x6BB7BD);
  	if (str == "darkolivegreen")         return(0x2F6B55);
  	if (str == "darkorange")             return(0x008CFF);
  	if (str == "darkorchid")             return(0xCC3299);
  	if (str == "darksalmon")             return(0x7A96E9);
  	if (str == "darkseagreen")           return(0x8BBC8F);
  	if (str == "darkslateblue")          return(0x8B3D48);
  	if (str == "darkslategray")          return(0x4F4F2F);
  	if (str == "darkturquoise")          return(0xD1CE00);
  	if (str == "darkviolet")             return(0xD30094);
  	if (str == "deeppink")               return(0x9314FF);
  	if (str == "deepskyblue")            return(0xFFBF00);
  	if (str == "dimgray")                return(0x696969);
  	if (str == "dodgerblue")             return(0xFF901E);
  	if (str == "firebrick")              return(0x2222B2);
  	if (str == "forestgreen")            return(0x228B22);
  	if (str == "gainsboro")              return(0xDCDCDC);
  	if (str == "gold")                   return(0x00D7FF);
  	if (str == "goldenrod")              return(0x20A5DA);
  	if (str == "gray")                   return(0x808080);
  	if (str == "green")                  return(0x008000);
  	if (str == "greenyellow")            return(0x2FFFAD);
  	if (str == "honeydew")               return(0xF0FFF0);
  	if (str == "hotpink")                return(0xB469FF);
  	if (str == "indianred")              return(0x5C5CCD);
  	if (str == "indigo")                 return(0x82004B);
  	if (str == "ivory")                  return(0xF0FFFF);
  	if (str == "khaki")                  return(0x8CE6F0);
  	if (str == "lavender")               return(0xFAE6E6);
  	if (str == "lavenderblush")          return(0xF5F0FF);
  	if (str == "lawngreen")              return(0x00FC7C);
  	if (str == "lemonchiffon")           return(0xCDFAFF);
  	if (str == "lightblue")              return(0xE6D8AD);
  	if (str == "lightcoral")             return(0x8080F0);
  	if (str == "lightcyan")              return(0xFFFFE0);
  	if (str == "lightgoldenrod")         return(0xD2FAFA);
  	if (str == "lightgray")              return(0xD3D3D3);
  	if (str == "lightgreen")             return(0x90EE90);
  	if (str == "lightpink")              return(0xC1B6FF);
  	if (str == "lightsalmon")            return(0x7AA0FF);
  	if (str == "lightseagreen")          return(0xAAB220);
  	if (str == "lightskyblue")           return(0xFACE87);
  	if (str == "lightslategray")         return(0x998877);
  	if (str == "lightsteelblue")         return(0xDEC4B0);
  	if (str == "lightyellow")            return(0xE0FFFF);
  	if (str == "lime")                   return(0x00FF00);
  	if (str == "limegreen")              return(0x32CD32);
  	if (str == "linen")                  return(0xE6F0FA);
  	if (str == "magenta")                return(0xFF00FF);
  	if (str == "maroon")                 return(0x000080);
  	if (str == "mediumaquamarine")       return(0xAACD66);
  	if (str == "mediumblue")             return(0xCD0000);
  	if (str == "mediumorchid")           return(0xD355BA);
  	if (str == "mediumpurple")           return(0xDB7093);
  	if (str == "mediumseagreen")         return(0x71B33C);
  	if (str == "mediumslateblue")        return(0xEE687B);
  	if (str == "mediumspringgreen")      return(0x9AFA00);
  	if (str == "mediumturquoise")        return(0xCCD148);
  	if (str == "mediumvioletred")        return(0x8515C7);
  	if (str == "midnightblue")           return(0x701919);
  	if (str == "mintcream")              return(0xFAFFF5);
  	if (str == "mistyrose")              return(0xE1E4FF);
  	if (str == "moccasin")               return(0xB5E4FF);
  	if (str == "navajowhite")            return(0xADDEFF);
  	if (str == "navy")                   return(0x800000);
  	if (str == "none")                   return(C'0x00,0x00,0x00');
  	if (str == "oldlace")                return(0xE6F5FD);
  	if (str == "olive")                  return(0x008080);
  	if (str == "olivedrab")              return(0x238E6B);
  	if (str == "orange")                 return(0x00A5FF);
  	if (str == "orangered")              return(0x0045FF);
  	if (str == "orchid")                 return(0xD670DA);
  	if (str == "palegoldenrod")          return(0xAAE8EE);
  	if (str == "palegreen")              return(0x98FB98);
  	if (str == "paleturquoise")          return(0xEEEEAF);
  	if (str == "palevioletred")          return(0x9370DB);
  	if (str == "papayawhip")             return(0xD5EFFF);
  	if (str == "peachpuff")              return(0xB9DAFF);
  	if (str == "peru")                   return(0x3F85CD);
  	if (str == "pink")                   return(0xCBC0FF);
  	if (str == "plum")                   return(0xDDA0DD);
  	if (str == "powderblue")             return(0xE6E0B0);
  	if (str == "purple")                 return(0x800080);
  	if (str == "red")                    return(0x0000FF);
  	if (str == "rosybrown")              return(0x8F8FBC);
  	if (str == "royalblue")              return(0xE16941);
  	if (str == "saddlebrown")            return(0x13458B);
  	if (str == "salmon")                 return(0x7280FA);
  	if (str == "sandybrown")             return(0x60A4F4);
  	if (str == "seagreen")               return(0x578B2E);
  	if (str == "seashell")               return(0xEEF5FF);
  	if (str == "sienna")                 return(0x2D52A0);
  	if (str == "silver")                 return(0xC0C0C0);
  	if (str == "skyblue")                return(0xEBCE87);
  	if (str == "slateblue")              return(0xCD5A6A);
  	if (str == "slategray")              return(0x908070);
  	if (str == "snow")                   return(0xFAFAFF);
  	if (str == "springgreen")            return(0x7FFF00);
  	if (str == "steelblue")              return(0xB48246);
  	if (str == "tan")                    return(0x8CB4D2);
  	if (str == "teal")                   return(0x808000);
  	if (str == "thistle")                return(0xD8BFD8);
  	if (str == "tomato")                 return(0x4763FF);
  	if (str == "turquoise")              return(0xD0E040);
  	if (str == "violet")                 return(0xEE82EE);
  	if (str == "wheat")                  return(0xB3DEF5);
  	if (str == "white")                  return(0xFFFFFF);
  	if (str == "whitesmoke")             return(0xF5F5F5);
  	if (str == "yellow")                 return(0x00FFFF);
  	if (str == "yellowgreen")            return(0x32CD9A);

  	int t1 = StringFind(str,",",0);
  	int t2 = StringFind(str,",",t1+1);
  	if (t1>0 && t2>0) {
  
    	int red   = StrToInteger(StringSubstr(str,0,t1));
    	int green = StrToInteger(StringSubstr(str,t1+1,t2-1));
    	int blue  = StrToInteger(StringSubstr(str,t2+1,StringLen(str)));
    	return(blue*256*256+green*256+red);
    
  	}  

  	return(0);
  
	}



	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// PRICE.ABRVTONAME
	// Converts price abbraviation to full price name

	string Price.AbrvToName(string p) {
	
		if(p=="o") return("OPEN");
		if(p=="c") return("CLOSE");
		if(p=="h") return("HIGH");
		if(p=="l") return("LOW");
		if(p=="m") return("MEDIAN");
		if(p=="t") return("TYPICAL");
	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// PRICE.ABRV
	// Converts full price name to single character abbreviation

	string Price.Abrv(string p) {

		if(p=="OPEN") 		return("o");
		if(p=="CLOSE") 	return("c");
		if(p=="HIGH") 		return("h");
		if(p=="LOW") 		return("l");
		if(p=="MEDIAN") 	return("m");
		if(p=="TYPICAL") 	return("t");
	
	}

//=====================================================================================================================================================================
//=====================================================================================================================================================================
//=====================================================================================================================================================================
// OBJECT UTILITIES	
 
	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// OBJECT.UNIQUENAME
	// Generates a unique name for a chart object by insertng a random number - use to make sure object does not already exist

	string Object.UniqueName(string prefix, string rest){
	
	  	string result = prefix+(MathRand()%1001)+rest;
   	while (WindowFind(result)> 0)  	result = prefix+(MathRand()%1001)+rest;
   	return(result);
   	
	}
	
	
//=====================================================================================================================================================================
//=====================================================================================================================================================================
//=====================================================================================================================================================================
// REVERSAL (RAT) UTILITIES	
 
	string revcolor = "REVCOLOR"; // colors:  RED for SHORT-ONLY TRADING, or GREEN for LONG-ONLY TRADING
	
	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// REV.GETCOLOR
	// Get the current GLOBAL "REVCOLOR", or set it to the passed in default color if it does not yet exist AND
	// the SetIfMissing flag is set to true - otherwise, return White as a "WTF?" error-type response

	color	Rev.GetColor(color defaultColor, bool SetIfMissing){
	
		if (GlobalVariableCheck(revcolor)) {
		
			 double rc=GlobalVariableGet(revcolor);
			 if (rc==1) return(Green); else if (rc==2) return(Red); else return(Yellow);
			 
		} else if (SetIfMissing) {
		
			Rev.SetColor(defaultColor);
			return(defaultColor);
			
		} else return(White);
	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// REV.SETCOLOR
	//FORCE set the color for the GLOBAL "REVCOLOR"	variable

	color	Rev.SetColor(color RevColor){
	
		if (RevColor==Green) GlobalVariableSet(revcolor,1);
		if (RevColor==Red) GlobalVariableSet(revcolor,2);
	
	} 
	
	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// REV.COLORCHANGED
	// Check if GLOBAL color is different from whatever is set in this indicator (without modifying the GLOBAL value)

	bool Rev.ColorChanged(color PreviousColor) {
	
		return (Rev.GetColor(PreviousColor,false)==PreviousColor);
	
	}

	//-----------------------------------------------------------------------------------------------------------------------------------------------------
	// REV.STATUS
	// REVERSAL STATUS (RAT TRADE RULES)
	// inputs:
	//				myPair 		= Currency pair being checked
	//				TimeFrame	= Chart Time Frame being checked
	//				Threshold	= How far price must have paced the trigger point to flag it as TRIGGERED
	//				ShortTrader	= Set to TRUE if you want to get back all SHORT signals, or FALSE to have them ignored (RED RAT)
	//				LongTrader	= Set to TRUE if you want to get back all LONG signals, or FALSE to have them ignored (GREEN RAT)
	//								  Setting both to TRUE means you get back ALL signals	
	// returns:
	//				0 = NOTHING TO SEE HERE, MOVE ALONG
	//				1 = SHORT TRADE SIGNAL HAS BEEN TRIGGED
	//				2 = LONG TRADE SIGNAL HAS BEEN TRIGGED
	//				3 = SHORT TRADE SETUP HAS FORMED - WAITING FOR TRIGGER
	//				4 = LONG TRADE SETUP HAS FORMED - WAITING FOR TRIGGER	
	
	int Rev.Status(string myPair, int TimeFrame, double Threshold, bool ShortTrader, bool LongTrader ){
	double C,O,H,L,O1,C1,H1,L1,O2,C2;

	bool PrevBarUp,Bar2Up,ThisBarUp;
		
      // CURRENT Candle OHLC
		C  = iClose(myPair,TimeFrame, 0); 
		O  = iOpen(myPair,TimeFrame, 0);          
		H  = iHigh(myPair,TimeFrame, 0);   
		L  = iLow(myPair,TimeFrame, 0);
		          
		// PREVIOUS Candle OHLC
		O1 = iOpen(myPair,TimeFrame, 1);           
		C1 = iClose(myPair,TimeFrame, 1);           
		H1 = iHigh(myPair,TimeFrame, 1);   
		L1 = iLow(myPair,TimeFrame, 1);
		    
		// TWO CANDLES BACK OPEN / CLOSE 
		O2 = iOpen(myPair,TimeFrame, 2);           
		C2 = iClose(myPair,TimeFrame, 2);

		ThisBarUp = C > O;   // is this a green-up bar or a red-down bar?
		PrevBarUp = C1 > O1; // was the previous candle an up or down bar?
		Bar2Up    = C2 > O2; // was the candle two bars ago an up or down bar? 

	
		if (C <= L1 && C >= L1 + Threshold && Bar2Up && !PrevBarUp  && ShortTrader  ) return(1); // Short Signal Triggered
		if (C >= H1 && C <= H1 + Threshold && LongTrader) 										return(2); // Long Signal Triggered
		if (Bar2Up && !PrevBarUp && ShortTrader ) 												return(3); // Short Setup in place - waiting for trigger
		if (!Bar2Up && PrevBarUp && LongTrader) 													return(4); // Long Setup in place - waiting for trigger    

		return(0);
   
	}